import pytest

from gso.products import ProductName
from gso.services.subscriptions import get_product_id_by_name
from gso.utils.shared_enums import APType
from test.workflows import assert_complete, run_workflow


@pytest.fixture()
def base_bgp_peer_input(faker):
    def _base_bgp_peer_input():
        bfd_enabled = faker.boolean()
        return {
            "bfd_enabled": bfd_enabled,
            "bfd_interval": faker.pyint() if bfd_enabled else None,
            "bfd_multiplier": faker.pyint() if bfd_enabled else None,
            "has_custom_policies": faker.boolean(),
            "authentication_key": faker.password(),
            "multipath_enabled": faker.boolean(),
            "send_default_route": faker.boolean(),
            "is_passive": faker.boolean(),
        }

    return _base_bgp_peer_input


@pytest.mark.workflow()
def test_create_geant_ip_success(
    responses, faker, partner_factory, edge_port_subscription_factory, base_bgp_peer_input
):
    partner = partner_factory(name=faker.company(), email=faker.email())
    product_id = get_product_id_by_name(ProductName.GEANT_IP)
    edge_port_a = edge_port_subscription_factory(partner=partner)

    form_input_data = [
        {"product": product_id},
        {"tt_number": faker.tt_number(), "partner": partner["partner_id"]},
        {"edge_ports": [{"edge_port": edge_port_a, "ap_type": APType.PRIMARY}]},
        {
            "geant_sid": faker.geant_sid(),
            "is_tagged": faker.boolean(),
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "custom_firewall_filters": faker.boolean(),
            "v4_bgp_peer": base_bgp_peer_input() | {"add_v4_multicast": faker.boolean(), "peer_address": faker.ipv4()},
            "v6_bgp_peer": base_bgp_peer_input() | {"add_v6_multicast": faker.boolean(), "peer_address": faker.ipv6()},
        },
    ]

    result, process_stat, step_log = run_workflow("create_geant_ip", form_input_data)
    assert process_stat, step_log
    assert_complete(result)
