from unittest.mock import patch

import pytest

from gso.products import Router
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
def test_redeploy_base_config_success(
    mock_provision_router,
    router_subscription_factory,
    faker,
):
    #  Set up mock return values
    product_id = router_subscription_factory()

    #  Run workflow
    initial_input_data = [{"subscription_id": product_id}, {"tt_number": faker.tt_number()}]
    result, process_stat, step_log = run_workflow("redeploy_base_config", initial_input_data)

    for _ in range(2):
        result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert mock_provision_router.call_count == 2
